import graphene
from datetime import datetime

from aleksis.apps.kolego.models import Absence

from .documentation import DocumentationType
from ..models import ParticipationStatus

class LessonsForPersonType(graphene.ObjectType):
    id = graphene.ID()  # noqa
    lessons = graphene.List(DocumentationType)

class AbsencesBatchCreateMutation(graphene.Mutation):
    class Arguments:
        persons = graphene.List(graphene.ID)
        start = graphene.Date(),
        end = graphene.Date(),
        comment  = graphene.String(),
        reason = graphene.ID()

    ok = graphene.Boolean()

    @classmethod
    def mutate(cls, root, info, persons, start, end, comment, reason):  # noqa
        # TODO: More permission checks needed?
        # DocumentationBatchCreateOrUpdateMutation.create_or_update
        # at least already checks permissions.

        for person in persons:
            # Get all documentations for this person between start & end
            # Could be shared with query
            event_params = {
                "type": "PARTICIPANT",
                "obj_id": PERSON_ID,
            }

            events = LessonEvent.get_single_events(
                datetime.combine(start, datetime.min.time()),
                datetime.combine(end, datetime.max.time()),
                None,
                event_params,
                with_reference_object=True,
            )

            docs, dummies = Documentation.get_documentations_for_events(events)
            # till here -> reuse?

            # Create doc for dummies that are already in the past
            future = false
            for dummy in dummies:
                # TODO/MAYBE: This in past logic could be somewhere else OR shared.
                # The next 5 lines are shared with DocumentationBatchCreateOrUpdateMutation
                # & could be deduplicated
                dummy, lesson_event_id, datetime_start_iso, datetime_end_iso = _id.split(";")
                lesson_event = LessonEvent.objects.get(id=lesson_event_id)
                start = datetime.fromisoformat(datetime_start_iso).astimezone(
                    lesson_event.timezone
                )

                if start < datetime.now():
                    # In the past -> Create a Documentation
                    docs.append(
                        DocumentationBatchCreateOrUpdateMutation.create_or_update(info, dummy)
                    )
                else:
                    future = true

            # Create a ParticipationStatus for each documentation
            for doc in docs:
                # TODO: Is ID for enough for person&reason OR should it
                # resolve to django object first?
                ParticipationStatus.objects.create(
                    person=person,
                    related_documentation=doc,
                    absence_reason=reason,
                )

            # If there are still dummy documentations in the future
            # create a Kolego Absence
            if future:
                # TODO: Should probably lookup person&reason as well
                # TODO: Are date_start & date_end from CalendarEvent enough
                #       or more needed?
                Absence.objects.create(
                    date_start=datetime.now().date(),
                    date_end=end,
                    reason=reason,
                    person=person,
                    comment=comment,
                )

        # Return ok=True if everything went well.
        return AbsencesBatchCreateMutation(ok=True)
